@@ -7,15 +7,10 @@
 #undef HAVE_LIBSM
 #undef HAVE_PROGRAM_INVOCATION_SHORT_NAME
 #undef HAVE_PROGRAM_INVOCATION_NAME
-#undef HAVE_BONOBO
 
-#undef GNOMECC_ICONS_DIR
-#undef GNOMECC_GLADE_DIR
-#undef GNOMECC_PIXMAPS_DIR
-#undef GNOMECC_DEFAULTS_DIR
-#undef GNOMECC_SCREENSAVERS_DIR
+#undef GNOMECC_DATA_DIR
 
-#undef PACKAGE
+#undef GETTEXT_PACKAGE
 #undef VERSION
 
 #undef HAVE_ESD
@@ -25,6 +20,9 @@
 #undef u_int64_t
 #undef int64_t
 
+/* Define if the system has Xft version 2 and fontconfig */
+#undef HAVE_XFT2
+
 #undef HAVE_DEVGTK
 
 #undef WITH_SYMBOL_UNDERSCORE
