@@ -42,12 +42,18 @@
 
 enum
 {
-	TARGET_URI_LIST
+	TARGET_URI_LIST,
+	TARGET_COLOR,
+	TARGET_BGIMAGE
+/*	TARGET_BACKGROUND_RESET*/  
 };
 
 static GtkTargetEntry drop_types[] =
 {
-	{"text/uri-list", 0, TARGET_URI_LIST}
+	{"text/uri-list", 0, TARGET_URI_LIST},
+	{ "application/x-color", 0, TARGET_COLOR },
+	{ "property/bgimage", 0, TARGET_BGIMAGE }
+/*	{ "x-special/gnome-reset-background", 0, TARGET_BACKGROUND_RESET }*/
 };
 
 static gint n_drop_types = sizeof (drop_types) / sizeof (GtkTargetEntry);
@@ -219,16 +225,16 @@ update_color_widget_labels_and_visibility (ApplierSet *set, const gchar *value_s
 
 	switch (orientation) {
 	case ORIENTATION_SOLID: /* solid */ 
-		color1_string = _("C_olor"); 
+		color1_string = _("C_olor:"); 
 		two_colors = FALSE; 
 		break;
 	case ORIENTATION_HORIZ: /* horiz */ 
-		color1_string = _("_Left Color"); 
-		color2_string = _("_Right Color"); 
+		color1_string = _("_Left Color:"); 
+		color2_string = _("_Right Color:"); 
 		break;
 	case ORIENTATION_VERT: /* vert  */
-		color1_string = _("_Top Color");
-		color2_string = _("_Bottom Color"); 
+		color1_string = _("_Top Color:");
+		color2_string = _("_Bottom Color:"); 
 		break;
 	default:
 		break;
@@ -424,7 +430,7 @@ create_dialog (ApplierSet *set)
 
 	g_slist_free (group);
 
-	label = gtk_label_new_with_mnemonic (_("_Picture"));
+	label = gtk_label_new_with_mnemonic (_("Select _picture:"));
 	gtk_frame_set_label_widget (GTK_FRAME (WID ("picture_frame")),
 				    label);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label),
@@ -441,19 +447,17 @@ create_dialog (ApplierSet *set)
 	return dialog;
 }
 
-/* Callback issued when a button is clicked on the dialog */
 
 static void
-dialog_button_clicked_cb (GtkDialog *dialog, gint response_id, GConfChangeSet *changeset) 
+cb_dialog_response (GtkDialog *dialog, gint response_id)
 {
-	switch (response_id) {
-	case GTK_RESPONSE_CLOSE:
-	case GTK_RESPONSE_DELETE_EVENT:
+	if (response_id == GTK_RESPONSE_HELP)
+		capplet_help (GTK_WINDOW (dialog),
+			"wgoscustdesk.xml",
+			"goscustdesk-7");
+	else
 		gtk_main_quit ();
-		break;
-	}
 }
-
 /* Callback issued during drag movements */
 
 static gboolean
@@ -480,40 +484,67 @@ drag_data_received_cb (GtkWidget *widget, GdkDragContext *context,
 		       GtkSelectionData *selection_data,
 		       guint info, guint time, gpointer data)
 {
-	GList *list;
-	GList *uris;
+	GConfClient *client = gconf_client_get_default ();
 	ApplierSet *set = (ApplierSet*) data; 
 
-	if (info != TARGET_URI_LIST)
-		return;
-	
-	uris = gnome_vfs_uri_list_parse ((gchar *) selection_data->
-					 data);
-	for (list = uris; list; list = list->next)
+	if (info == TARGET_URI_LIST ||
+	    info == TARGET_BGIMAGE)
 	{
-		GnomeVFSURI *uri = (GnomeVFSURI *) list->data;
-		GConfEntry *entry;
-		GConfValue *value = gconf_value_new (GCONF_VALUE_STRING);
-		GConfClient *client = gconf_client_get_default ();
+		GList *uris;
+
+		g_print ("got %s\n", selection_data->data);
+		uris = gnome_vfs_uri_list_parse ((gchar *) selection_data->data);
+		/* Arbitrarily pick the first item */
+		if (uris != NULL && uris->data != NULL)
+		{
+			GnomeVFSURI *uri = (GnomeVFSURI *) uris->data;
+			GConfEntry *entry;
+			GConfValue *value = gconf_value_new (GCONF_VALUE_STRING);
+			gchar *base;
+
+			base = gnome_vfs_unescape_string (gnome_vfs_uri_get_path (uri),
+							  G_DIR_SEPARATOR_S);
+			gconf_value_set_string (value, base);
+			g_free (base);
+
+			/* Hmm, should we bother with changeset here? */
+			gconf_client_set (client, BG_PREFERENCES_PICTURE_FILENAME, value, NULL);
+			gconf_client_suggest_sync (client, NULL);
+
+			/* this isn't emitted by the peditors,
+			 * so we have to manually update */
+			/* FIXME: this can't be right!!!! -jrb */
+			entry = gconf_entry_new (BG_PREFERENCES_PICTURE_FILENAME, value);
+			bg_preferences_merge_entry (set->prefs, entry);
+			gconf_entry_free (entry);
+			gconf_value_free (value);
+
+			gconf_client_set_string (client, BG_PREFERENCES_PICTURE_OPTIONS, "wallpaper", NULL);
+		}
+		gnome_vfs_uri_list_free (uris);
+	} else if (info == TARGET_COLOR) {
+		guint16 *channels;
+		char *color_spec;
 		
-		gconf_value_set_string (value, gnome_vfs_uri_get_path (uri));
+		if (selection_data->length != 8 || selection_data->format != 16) {
+			return;
+		}
+        
+		channels = (guint16 *) selection_data->data;
+		color_spec = g_strdup_printf ("#%02X%02X%02X", channels[0] >> 8, channels[1] >> 8, channels[2] >> 8);
+		gconf_client_set_string (client, BG_PREFERENCES_PICTURE_OPTIONS, "none", NULL);
+		gconf_client_set_string (client, BG_PREFERENCES_COLOR_SHADING_TYPE, "solid", NULL);
+		gconf_client_set_string (client, BG_PREFERENCES_PRIMARY_COLOR, color_spec, NULL);
+		g_free (color_spec);
+	} else if (info == TARGET_BGIMAGE) {
 
-		/* Hmm, should we bother with changeset here? */
-		gconf_client_set (client, BG_PREFERENCES_PICTURE_FILENAME, value, NULL);
-		gconf_client_suggest_sync (client, NULL);
+	} else if (info == TARGET_URI_LIST) {
 
-		/* this isn't emitted by the peditors,
-		 * so we have to manually update */
-		entry = gconf_entry_new (BG_PREFERENCES_PICTURE_FILENAME, value);
-		bg_preferences_merge_entry (set->prefs, entry);
-		gconf_entry_free (entry);
-		gconf_value_free (value);
 	}
-
+	
 	if (GTK_WIDGET_REALIZED (bg_applier_get_preview_widget (set->appliers[n_enum_vals - 1])))
 		applier_set_redraw (set);
 
-	gnome_vfs_uri_list_free (uris);
 }
 
 int
@@ -523,7 +554,6 @@ main (int argc, char **argv)
 	GladeXML       *dialog;
 	GtkWidget      *dialog_win;
 	ApplierSet     *set;
-	GdkPixbuf      *pixbuf;
 
 	static gboolean get_legacy;
 	static struct poptOption cap_options[] = {
@@ -536,7 +566,8 @@ main (int argc, char **argv)
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	gnome_program_init (argv[0], VERSION, LIBGNOMEUI_MODULE, argc, argv,
+	gnome_program_init ("gnome-background-properties", VERSION,
+			    LIBGNOMEUI_MODULE, argc, argv,
 			    GNOME_PARAM_POPT_TABLE, cap_options,
 			    NULL);
 
@@ -553,15 +584,16 @@ main (int argc, char **argv)
 		setup_dialog (dialog, NULL, set);
 
 		dialog_win = gtk_dialog_new_with_buttons
-			(_("Background Preferences"), NULL, -1,
+			(_("Background Preferences"), NULL, 0,
+			 GTK_STOCK_HELP, GTK_RESPONSE_HELP,
 			 GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 			 NULL);
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog_win),
+			GTK_RESPONSE_CLOSE);
 
-		pixbuf = gdk_pixbuf_new_from_file (GNOMECC_DATA_DIR "/icons/background-capplet.png", NULL);
-		gtk_window_set_icon (GTK_WINDOW(dialog_win), pixbuf);
-		gdk_pixbuf_unref (pixbuf);
-
-		g_signal_connect (G_OBJECT (dialog_win), "response", (GCallback) dialog_button_clicked_cb, NULL);
+		g_signal_connect (G_OBJECT (dialog_win),
+				    "response",
+				    G_CALLBACK (cb_dialog_response), NULL);
 
 		gtk_drag_dest_set (dialog_win, GTK_DEST_DEFAULT_ALL,
 				   drop_types, n_drop_types,
@@ -576,6 +608,7 @@ main (int argc, char **argv)
 
 		g_object_weak_ref (G_OBJECT (dialog_win), (GWeakNotify) applier_set_free, set);
 		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_win)->vbox), WID ("prefs_widget"), TRUE, TRUE, GNOME_PAD_SMALL);
+		capplet_set_icon (dialog_win, "background-capplet.png");
 		gtk_widget_show (dialog_win);
 
 		gtk_main ();
